@@ -152,10 +152,14 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
     }
 
     URL uri (str);
-    if (mime.isEmpty () && uri.isLocalFile ()) {
-        setMimetype (MimeType::findByURL (uri));
-        if (mrl)
-            mrl->mimetype = mime;
+    if (uri.isLocalFile ()) {
+        if (mime.isEmpty ()) {
+            setMimetype (MimeType::findByURL (uri));
+            if (mrl)
+                mrl->mimetype = mime;
+        } else {
+            setMimetype (mime);
+        }
     }
 
     bool only_playlist = false;
@@ -219,8 +223,11 @@ KDE_NO_EXPORT bool MediaInfo::wget (const String &str) {
     }
 
     String protocol = uri.protocol ();
-    if (protocol == "mms" || protocol == "rtsp" || protocol == "rtp" ||
+    if (protocol == "mms" || protocol == "rtsp" ||
+            protocol == "rtp" || protocol == "rtmp" ||
             (only_playlist && !maybe_playlist && !mime.isEmpty ())) {
+        if (ActorAgent::Any == type)
+            type = ActorAgent::AudioVideo;
         ready ();
         return true;
     }
