@@ -45,10 +45,16 @@ struct EvalState {
     void addRef () { ++ref_count; }
     void removeRef () { if (--ref_count == 0) delete this; }
 
+    String value () const {
+        if (attr)
+            return attr->value ();
+        return root->nodeValue ();
+    }
+
     String def_root_tag;
     Node *root;
     Attribute *attr;
-    NodeValueList *process_list;
+    Sequence *process_list;
     EvalState *parent;
     int sequence;
     int ref_count;
@@ -66,7 +72,7 @@ struct AST : public Expression {
     virtual int toInt () const;
     virtual float toFloat () const;
     virtual String toString () const;
-    virtual NodeValueList *toNodeList () const; // uncachable
+    virtual Sequence *toSequence () const; // uncachable
     virtual Type type () const;
     virtual void setRoot (Node *root);
     void setRoot (Node *root, Attribute *a);
@@ -157,7 +163,6 @@ struct Step : public StringBase {
 
     bool matches (Node *n);
     bool matches (Attribute *a);
-    bool selected (Node *n, Attribute *a);
     bool anyPath () const { return string.isEmpty (); }
 #ifdef KMPLAYER_EXPR_DEBUG
     virtual void dump () const {
@@ -175,15 +180,20 @@ struct Step : public StringBase {
     bool start_contextual;
 };
 
-struct Identifier : public StringBase {
-    Identifier (EvalState *ev, AST *steps) : StringBase (ev) {
-        first_child = steps;
-    }
+struct SequenceBase : public StringBase {
+    SequenceBase (EvalState *ev) : StringBase (ev) {}
 
     virtual bool toBool () const;
     virtual String toString () const;
-    virtual NodeValueList *toNodeList () const;
     virtual Type type () const;
+};
+
+struct Identifier : public SequenceBase {
+    Identifier (EvalState *ev, AST *steps) : SequenceBase (ev) {
+        first_child = steps;
+    }
+
+    virtual Sequence *toSequence () const;
     void childByStep (Step *step, bool recurse) const;
     void childByPath (Step *step, bool recurse) const;
 #ifdef KMPLAYER_EXPR_DEBUG
@@ -220,6 +230,18 @@ struct Not : public BoolBase {
     virtual bool toBool () const;
 };
 
+struct StartsWith: public BoolBase {
+    StartsWith (EvalState *ev) : BoolBase (ev) {}
+
+    virtual bool toBool () const;
+};
+
+struct Count : public IntegerBase {
+    Count (EvalState *ev) : IntegerBase (ev) {}
+
+    virtual int toInt () const;
+};
+
 struct HoursFromTime : public IntegerBase {
     HoursFromTime (EvalState *ev) : IntegerBase (ev) {}
 
@@ -256,6 +278,12 @@ struct Position : public IntegerBase {
     virtual int toInt () const;
 };
 
+struct StringLength : public IntegerBase {
+    StringLength (EvalState *ev) : IntegerBase (ev) {}
+
+    virtual int toInt () const;
+};
+
 struct Concat : public StringBase {
     Concat (EvalState *ev) : StringBase (ev) {}
 
@@ -268,6 +296,18 @@ struct StringJoin : public StringBase {
     virtual String toString () const;
 };
 
+struct SubstringAfter : public StringBase {
+    SubstringAfter (EvalState *ev) : StringBase (ev) {}
+
+    virtual String toString () const;
+};
+
+struct SubstringBefore : public StringBase {
+    SubstringBefore (EvalState *ev) : StringBase (ev) {}
+
+    virtual String toString () const;
+};
+
 struct CurrentTime : public StringBase {
     CurrentTime (EvalState *ev) : StringBase (ev) {}
 
@@ -286,10 +326,10 @@ struct EscapeUri : public StringBase {
     virtual String toString () const;
 };
 
-struct Sort : public AST {
-    Sort (EvalState *ev) : AST (ev) {}
+struct Sort : public SequenceBase {
+    Sort (EvalState *ev) : SequenceBase (ev) {}
 
-    virtual NodeValueList *toNodeList () const;
+    virtual Sequence *toSequence () const;
 };
 
 struct Multiply : public NumberBase {
@@ -357,12 +397,12 @@ struct Minus : public NumberBase {
 #endif
 };
 
-struct Join : public AST {
-    Join (EvalState *ev, AST *children) : AST (ev) {
+struct Join : public SequenceBase {
+    Join (EvalState *ev, AST *children) : SequenceBase (ev) {
         first_child = children;
     }
 
-    virtual NodeValueList *toNodeList () const;
+    virtual Sequence *toSequence () const;
 #ifdef KMPLAYER_EXPR_DEBUG
     virtual void dump () const;
 #endif
@@ -427,8 +467,8 @@ String AST::toString () const {
     }
 }
 
-NodeValueList *AST::toNodeList () const {
-    return new NodeValueList;
+Sequence *AST::toSequence () const {
+    return new Sequence;
 }
 
 AST::Type AST::type () const {
@@ -491,7 +531,7 @@ AST::Type BoolBase::type () const {
 bool NumberBase::toBool () const {
     int ii = toInt ();
     if (eval_state->parent) {
-        NodeValueList *lst = eval_state->parent->process_list;
+        Sequence *lst = eval_state->parent->process_list;
         if (lst) {
             int count = 0;
             for (NodeValueItem *n = lst->first (); n; n = n->nextSibling ())
@@ -558,19 +598,11 @@ bool Step::matches (Attribute *a) {
     return any_node || attr == a->name ();
 }
 
-bool Step::selected (Node *n, Attribute *a) {
-    if (first_child) {
-        first_child->setRoot (n, a);
-        return first_child->toBool ();
-    }
-    return true;
-}
-
-bool Identifier::toBool () const {
+bool SequenceBase::toBool () const {
     bool b = false;
     if (eval_state->parent) {
         sequence = eval_state->sequence;
-        NodeValueList *lst = toNodeList ();
+        Sequence *lst = toSequence ();
         b = lst && lst->first ();
         delete lst;
     } else {
@@ -579,10 +611,39 @@ bool Identifier::toBool () const {
     return b;
 }
 
+String SequenceBase::toString () const {
+    if (eval_state->sequence != sequence) {
+        Sequence *lst = toSequence ();
+        int i = lst->length ();
+        if (i == 1)
+            string = lst->first ()->data.value ();
+        else
+            string = String::number (i);
+        delete lst;
+        sequence = eval_state->sequence;
+    }
+    return string;
+}
+
+AST::Type SequenceBase::type () const {
+    String s = toString ();
+    if (s.lower () == "true" ||
+            s.lower () == "false")
+        return TBool;
+    bool ok;
+    s.toInt (&ok);
+    if (ok)
+        return TInteger;
+    s.toDouble (&ok);
+    if (ok)
+        return TFloat;
+    return TString;
+}
+
 void Identifier::childByStep (Step *step, bool recurse) const {
-    NodeValueList *lst = eval_state->process_list;
+    Sequence *lst = eval_state->process_list;
     NodeValueItem *last = lst->last ();
-    NodeValueList recursive_lst;
+    Sequence recursive_lst;
     for (NodeValueItem *itm = lst->first (); itm; ) {
         NodeValueItem *next = itm == last ? NULL : itm->nextSibling ();
 
@@ -623,37 +684,27 @@ void Identifier::childByStep (Step *step, bool recurse) const {
 void Identifier::childByPath (Step *step, bool recurse) const {
     if (!step->anyPath ()) {
         childByStep (step, recurse);
-        NodeValueItem *itm = eval_state->process_list->first ();
-        if (itm && step->first_child) {
-            NodeValueList *newlist = new NodeValueList;
-            for (; itm; itm = itm->nextSibling ())
-                if (step->selected (itm->data.node, itm->data.attr))
-                    newlist->append (new NodeValueItem (itm->data));
-            delete eval_state->process_list;
-            eval_state->process_list = newlist;
+        for (AST *pred = step->first_child; pred; pred = pred->next_sibling) {
+            NodeValueItem *itm = eval_state->process_list->first ();
+            if (itm) {
+                Sequence *newlist = new Sequence;
+                for (; itm; itm = itm->nextSibling ()) {
+                    pred->setRoot (itm->data.node, itm->data.attr);
+                    if (pred->toBool ())
+                        newlist->append (new NodeValueItem (itm->data));
+                }
+                delete eval_state->process_list;
+                eval_state->process_list = newlist;
+            }
         }
     }
     if (step->next_sibling)
         childByPath ((Step *) step->next_sibling, step->anyPath ());
 }
 
-String Identifier::toString () const {
-    if (eval_state->sequence != sequence) {
-        NodeValueList *lst = toNodeList ();
-        int i = lst->length ();
-        if (i == 1)
-            string = lst->first ()->data.value ();
-        else
-            string = String::number (i);
-        delete lst;
-        sequence = eval_state->sequence;
-    }
-    return string;
-}
-
-NodeValueList *Identifier::toNodeList () const {
-    NodeValueList *old = eval_state->process_list;
-    NodeValueList *lst = new NodeValueList;
+Sequence *Identifier::toSequence () const {
+    Sequence *old = eval_state->process_list;
+    Sequence *lst = new Sequence;
     eval_state->process_list = lst;
     EvalState *es = eval_state;
     if (!((Step *)first_child)->start_contextual)
@@ -667,21 +718,6 @@ NodeValueList *Identifier::toNodeList () const {
     return lst;
 }
 
-AST::Type Identifier::type () const {
-    String s = toString ();
-    if (s.lower () == "true" ||
-            s.lower () == "false")
-        return TBool;
-    bool ok;
-    s.toInt (&ok);
-    if (ok)
-        return TInteger;
-    s.toDouble (&ok);
-    if (ok)
-        return TFloat;
-    return TString;
-}
-
 String StringLiteral::toString () const {
     return string;
 }
@@ -711,6 +747,35 @@ bool Not::toBool () const {
     return b;
 }
 
+bool StartsWith::toBool () const {
+    if (eval_state->sequence != sequence) {
+        sequence = eval_state->sequence;
+        b = false;
+        if (first_child) {
+            AST *s = first_child->next_sibling;
+            if (s)
+                b = first_child->toString ().startsWith (s->toString ());
+            else if (eval_state->parent)
+                b = eval_state->value ().startsWith (first_child->toString ());
+        }
+    }
+    return b;
+}
+
+int Count::toInt () const {
+    if (eval_state->sequence != sequence) {
+        sequence = eval_state->sequence;
+        if (first_child) {
+            Sequence *lst = first_child->toSequence ();
+            i = lst->length ();
+            delete lst;
+        } else {
+            i = 0;
+        }
+    }
+    return i;
+}
+
 int HoursFromTime::toInt () const {
     if (eval_state->sequence != sequence) {
         if (first_child) {
@@ -763,7 +828,7 @@ int Last::toInt () const {
     if (eval_state->sequence != sequence) {
         sequence = eval_state->sequence;
         if (eval_state->parent) {
-            NodeValueList *lst = eval_state->parent->process_list;
+            Sequence *lst = eval_state->parent->process_list;
             if (lst)
                 i = lst->length ();
         }
@@ -784,7 +849,7 @@ int Position::toInt () const {
     if (eval_state->sequence != sequence) {
         sequence = eval_state->sequence;
         if (eval_state->parent) {
-            NodeValueList *lst = eval_state->parent->process_list;
+            Sequence *lst = eval_state->parent->process_list;
             Node *r = eval_state->root;
             if (lst) {
                 i = 0;
@@ -799,6 +864,19 @@ int Position::toInt () const {
     return i;
 }
 
+int StringLength::toInt () const {
+    if (eval_state->sequence != sequence) {
+        sequence = eval_state->sequence;
+        if (first_child)
+            i = first_child->toString ().length ();
+        else if (eval_state->parent)
+            i = eval_state->value ().length ();
+        else
+            i = 0;
+    }
+    return i;
+}
+
 String Concat::toString () const {
     if (eval_state->sequence != sequence) {
         sequence = eval_state->sequence;
@@ -825,7 +903,7 @@ String StringJoin::toString () const {
         string.clear ();
         AST *child = first_child;
         if (child) {
-            NodeValueList *lst = child->toNodeList ();
+            Sequence *lst = child->toSequence ();
             NodeValueItem *n = lst->first();
             if (n) {
                 String sep;
@@ -841,6 +919,44 @@ String StringJoin::toString () const {
     return string;
 }
 
+String SubstringAfter::toString () const {
+    if (eval_state->sequence != sequence) {
+        sequence = eval_state->sequence;
+        string.clear ();
+        AST *child = first_child;
+        if (child) {
+            AST *next = child->next_sibling;
+            if (next) {
+                String s = child->toString ();
+                String t = next->toString ();
+                int p = s.indexOf (t);
+                if (p > -1)
+                    string = s.mid (p + t.length ());
+            }
+        }
+    }
+    return string;
+}
+
+String SubstringBefore::toString () const {
+    if (eval_state->sequence != sequence) {
+        sequence = eval_state->sequence;
+        string.clear ();
+        AST *child = first_child;
+        if (child) {
+            AST *next = child->next_sibling;
+            if (next) {
+                String s = child->toString ();
+                String t = next->toString ();
+                int p = s.indexOf (t);
+                if (p > -1)
+                    string = s.left (p);
+            }
+        }
+    }
+    return string;
+}
+
 String CurrentTime::toString () const {
     if (eval_state->sequence != sequence) {
         char buf[200];
@@ -865,10 +981,10 @@ String CurrentDate::toString () const {
     return string;
 }
 
-static void sortList (NodeValueList *lst, Expression *expr) {
+static void sortList (Sequence *lst, Expression *expr) {
     NodeValueItem *cur = lst->first ();
-    NodeValueList lt;
-    NodeValueList gt;
+    Sequence lt;
+    Sequence gt;
     expr->setRoot (cur->data.node);
     String str = expr->toString ();
     for (NodeValueItem *itm = cur->nextSibling (); itm; ) {
@@ -896,12 +1012,12 @@ static void sortList (NodeValueList *lst, Expression *expr) {
     }
 }
 
-NodeValueList *Sort::toNodeList () const {
+Sequence *Sort::toSequence () const {
     if (first_child) {
         Expression *exp = evaluateExpr (first_child->toString ());
         if (exp) {
             exp->setRoot (eval_state->root);
-            NodeValueList *lst = exp->toNodeList ();
+            Sequence *lst = exp->toSequence ();
             if (lst->first () && first_child->next_sibling) {
                 Expression *sort_exp =
                     evaluateExpr (first_child->next_sibling->toString ());
@@ -914,7 +1030,7 @@ NodeValueList *Sort::toNodeList () const {
             return lst;
         }
     }
-    return AST::toNodeList ();
+    return AST::toSequence ();
 }
 
 #define BIN_OP_TO_INT(NAME,OP)                                           \
@@ -1053,17 +1169,17 @@ void Minus::dump () const {
 }
 #endif
 
-NodeValueList *Join::toNodeList () const {
+Sequence *Join::toSequence () const {
     if (first_child) {
-        NodeValueList *lst = first_child->toNodeList ();
+        Sequence *lst = first_child->toSequence ();
         if (first_child->next_sibling) {
-            NodeValueList *l2 = first_child->next_sibling->toNodeList ();
+            Sequence *l2 = first_child->next_sibling->toSequence ();
             lst->splice (NULL, *l2);
             delete l2;
         }
         return lst;
     }
-    return AST::toNodeList ();
+    return AST::toSequence ();
 }
 
 #ifdef KMPLAYER_EXPR_DEBUG
@@ -1243,16 +1359,31 @@ static bool parseStep (const char *str, const char **end, AST *ast) {
         if (str == s)
             return false;
         entry = new Step (ast->eval_state, str, s, is_attr);
-        if (*s == '[') {
+        while (*s == '[') {
             AST pred (new EvalState (ast->eval_state));
             if (parseStatement (s + 1, end, &pred)) {
                 str = *end;
                 if (parseSpace (str, end))
                     str = *end;
                 if (*str == ']') {
-                    entry->first_child = pred.first_child;
-                    pred.first_child = NULL;
+                    if (pred.first_child) {
+                        appendASTChild (entry, pred.first_child);
+                        pred.first_child = NULL;
+                    }
                     s = ++str;
+                } else {
+                    delete entry;
+                    return false;
+                }
+            } else {
+                str = s + 1;
+                if (parseSpace (str, end))
+                    str = *end;
+                if (*str == ']') {
+                    s = ++str;
+                } else {
+                    delete entry;
+                    return false;
                 }
             }
         }
@@ -1372,6 +1503,8 @@ static bool parseFunction (const char *str, const char **end, AST *ast) {
                     func = new Concat (ast->eval_state);
                 else if (name == "contains")
                     func = new Contains (ast->eval_state);
+                else if (name == "count")
+                    func = new Count (ast->eval_state);
                 else if (name == "hours-from-time")
                     func = new HoursFromTime (ast->eval_state);
                 else if (name == "minutes-from-time")
@@ -1392,8 +1525,16 @@ static bool parseFunction (const char *str, const char **end, AST *ast) {
                     func = new Position (ast->eval_state);
                 else if (name == "sort")
                     func = new Sort (ast->eval_state);
+                else if (name == "starts-with")
+                    func = new StartsWith (ast->eval_state);
                 else if (name == "string-join")
                     func = new StringJoin (ast->eval_state);
+                else if (name == "string-length")
+                    func = new StringLength (ast->eval_state);
+                else if (name == "substring-after")
+                    func = new SubstringAfter (ast->eval_state);
+                else if (name == "substring-before")
+                    func = new SubstringBefore (ast->eval_state);
                 else if (name == "escape-uri")
                     func = new EscapeUri (ast->eval_state);
                 else
