@@ -133,9 +133,11 @@ public:
     struct DurationItem {
         DurationItem ();
         DurationItem &operator = (const DurationItem &other);
+        bool matches (const Duration dur, const Posting *post);
         void clear();
         Duration durval;
         int offset;
+        VirtualVoid *payload;
         ConnectionLink connection;
         DurationItem *next;
     } durations [(const int) DurTimeLast];
@@ -294,6 +296,7 @@ const short id_node_text_styling = 126;
 const short id_node_set_value = 127;
 const short id_node_new_value = 128;
 const short id_node_del_value = 129;
+const short id_node_send = 130;
 const short id_node_set = 132;
 const short id_node_animate = 133;
 const short id_node_animate_color = 134;
@@ -360,6 +363,8 @@ public:
 class KMPLAYER_NO_EXPORT State : public Element {
 public:
     enum Where { before, after, child };
+    enum Method { get, put };
+    enum Replace { all, instance, none };
 
     State (NodePtr & d);
 
@@ -375,6 +380,7 @@ public:
     void newValue (Node *ref, Where w, const String &name, const String &val);
     void setValue (Node *ref, const String &value);
     void delValue (Node *ref);
+    void stateChanged (Node *ref);
 
     ConnectionList m_StateChangeListeners;        // setValue changed a value
     PostponePtr postpone_lock;                    // pause while loading src
@@ -886,6 +892,24 @@ public:
     KDE_NO_EXPORT const char *nodeName () const { return "delvalue"; }
 };
 
+class KMPLAYER_NO_EXPORT Send : public StateValue {
+public:
+    Send (NodePtr &d) : StateValue (d, id_node_send), media_info (NULL) {}
+
+    virtual void init ();
+    virtual void begin ();
+    virtual void deactivate ();
+    virtual void parseParam (const TrieString &name, const String &value);
+    virtual void message (MessageType msg, void *content=NULL);
+    KDE_NO_EXPORT const char *nodeName () const { return "send"; }
+
+private:
+    String action;
+    SMIL::State::Replace replace;
+    SMIL::State::Method method;
+    MediaInfo *media_info;
+};
+
 class KMPLAYER_NO_EXPORT AnimateGroup : public Element {
 public:
     ~AnimateGroup ();
