@@ -1457,8 +1457,10 @@ bool DocumentBuilder::startTag(const String &tag, const AttributeList &attr) {
     if (m_ignore_depth) {
         m_ignore_depth++;
         //kdDebug () << "Warning: ignored tag " << tag.latin1 () << " ignore depth = " << m_ignore_depth << endl;
+    } else if (!m_node) {
+        return false; // had underflow
     } else {
-        NodePtr n = m_node ? m_node->childFromTag (tag) : NULL;
+        NodePtr n = m_node->childFromTag (tag);
         if (!n) {
             //kdDebug () << "Warning: unknown tag " << tag.latin1 () << endl;
             NodePtr doc = m_root->document ();
@@ -1486,6 +1488,8 @@ bool DocumentBuilder::endTag (const String & tag) {
     if (m_ignore_depth) { // endtag to ignore
         m_ignore_depth--;
         //kdDebug () << "Warning: ignored end tag " << " ignore depth = " << m_ignore_depth <<  endl;
+    } else if (!m_node) {
+        return false; // had underflow
     } else {  // endtag
         NodePtr n = m_node;
         while (n) {
@@ -1543,16 +1547,16 @@ bool DocumentBuilder::characterData (const String &data, bool append_only) {
         }
     }
     //kdDebug () << "characterData " << d.latin1() << endl;
-    return true;
+    return !!m_node;
 }
 
 bool DocumentBuilder::cdataData (const String & data) {
-    if (!m_ignore_depth) {
+    if (!m_ignore_depth && m_node) {
         NodePtr d = m_node->document ();
         m_node->appendChild (new CData (d, data));
     }
     //kdDebug () << "characterData " << d.latin1() << endl;
-    return true;
+    return !!m_node;
 }
 
 #ifdef HAVE_EXPAT
@@ -1993,13 +1997,13 @@ bool SimpleSAXParser::readAttributes () {
                   kdDebug () << "encodeing " << i.data().latin1() << endl;*/
         }
     } else {
-        have_error = builder.startTag (tagname, m_attributes);
+        have_error = !builder.startTag (tagname, m_attributes);
         if (closed)
-            have_error &= builder.endTag (tagname);
+            have_error &= !builder.endTag (tagname);
         //kdDebug () << "readTag " << tagname << " closed:" << closed << " ok:" << have_error << endl;
     }
     m_state = m_state->next; // pop Node or PI
-    return true;
+    return !have_error;
 }
 
 bool SimpleSAXParser::readPI () {
@@ -2053,7 +2057,7 @@ bool SimpleSAXParser::readCDATA () {
             cdata.truncate (cdata.length () - 2);
             m_state = m_state->next;
             if (m_state->state == InContent)
-                have_error = builder.cdataData (cdata);
+                have_error = !builder.cdataData (cdata);
             else if (m_state->state == InAttributes) {
                 if (equal_seen)
                     attr_value += cdata;
@@ -2089,7 +2093,7 @@ bool SimpleSAXParser::readEndTag () {
         if (!nextToken ()) return false;
     if (token->token != tok_angle_close)
         return false;
-    have_error = builder.endTag (tagname);
+    have_error = !builder.endTag (tagname);
     m_state = m_state->next;
     return true;
 }
@@ -2173,10 +2177,10 @@ bool SimpleSAXParser::parse (TextStream & d) {
                                 if (pos > -1)
                                     white_space = white_space.mid (pos + 1);
                             }
-                            have_error = builder.characterData (white_space);
+                            have_error = !builder.characterData (white_space);
                             white_space.truncate (0);
                         }
-                        have_error = builder.characterData (token->string);
+                        have_error = !builder.characterData (token->string);
                         in_character_data = true;
                     }
                 }
