@@ -690,8 +690,8 @@ struct KMPLAYER_NO_EXPORT Command {
  **/
 class KMPLAYER_NO_EXPORT Posting : public Command {
 public:
-    KDE_NO_CDTOR_EXPORT Posting (Node *n, Node *tgt, MessageType msg)
-        : source (n), target (tgt), message (msg) {}
+    KDE_NO_CDTOR_EXPORT Posting (Node *n, Node *tgt, MessageType msg, VirtualVoid *p=NULL)
+        : source (n), target (tgt), message (msg), payload (p) {}
     KDE_NO_CDTOR_EXPORT virtual ~Posting () {}
 
     virtual void apply (struct timeval &timeout);
@@ -702,6 +702,7 @@ public:
     NodePtrW source;
     NodePtrW target;
     MessageType message;
+    VirtualVoid *payload;
 };
 
 /**
