@@ -396,7 +396,7 @@ MPlayer::~MPlayer () {
 
 KMPLAYER_NO_EXPORT bool MPlayer::play (const String &uri) {
     if (!needs_restarted && playing ()) {
-        if (m_mrl_notify ||
+        if (m_mrl_notify &&
                 (m_transition_state == Paused ||
                  (Paused == m_state && m_transition_state != Playing))) {
             m_transition_state = Playing;
