@@ -322,12 +322,14 @@ int GLibString::indexOf (const GLibString &needle, int offset) const {
             data->buf_len = strlen (data->ptr);
         if (offset >= data->buf_len)
             return -1;
-        p += offset;
+        p = g_utf8_offset_to_pointer (data->ptr, offset);
+        if (!p)
+            return -1;
     }
     gchar * str = strstr (p, needle.data->ptr);
     if (!str)
         return -1;
-    return int (str - data->ptr);
+    return g_utf8_strlen (p, str - p) + (offset > -1 ? offset : 0);
 }
 
 int GLibString::lastIndexOf (const GLibString & needle) const {
@@ -338,7 +340,7 @@ int GLibString::lastIndexOf (const GLibString & needle) const {
     gchar * str = g_strrstr (data->ptr, needle.data->ptr);
     if (!str)
         return -1;
-    return int (str - data->ptr);
+    return g_utf8_strlen (data->ptr, str - data->ptr);
 }
 
 GLibString GLibString::replace (Char c1, Char c2) const {
