@@ -40,6 +40,13 @@ int main() {
         TEST_VERIFY (expr->toInt () == 3);
         delete expr;
     }
+    {
+        TEST_INFO (Test count(/books/book) == 3);
+        Expression *expr = evaluateExpr ("count(/books/book)");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toInt () == 3);
+        delete expr;
+    }
     {
         TEST_INFO (Test "//book == 3");
         Expression *expr = evaluateExpr ("//book");
@@ -51,11 +58,25 @@ int main() {
         TEST_INFO (Test "/books/book[@price='5$'] == 1");
         Expression *expr = evaluateExpr ("/books/book[@price='5$']");
         expr->setRoot (books);
-        NodeValueList *lst = expr->toNodeList ();
+        Sequence *lst = expr->toSequence ();
         TEST_VERIFY (lst->length () == 1);
         delete lst;
         delete expr;
     }
+    {
+        TEST_INFO (Test "/books/book[@price][2]/title == 'Pygmalion'");
+        Expression *expr = evaluateExpr ("/books/book[@price][2]/title");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Pygmalion");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test "/books/book[@price][2][]/title == 'Pygmalion'");
+        Expression *expr = evaluateExpr ("/books/book[@price][2][]/title");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Pygmalion");
+        delete expr;
+    }
     {
         TEST_INFO (Test "@price of third book] == '5$'");
         Expression *expr = evaluateExpr ("/books/book[3]/@price");
@@ -81,7 +102,7 @@ int main() {
         TEST_INFO (Test "@price == '5$' using last book as root");
         Expression *expr = evaluateExpr ("/books/book[last()]");
         expr->setRoot (books);
-        NodeValueList *lst = expr->toNodeList ();
+        Sequence *lst = expr->toSequence ();
         TEST_VERIFY (lst->first ());
         if (lst->first ()) {
             NodePtr b3 = lst->first ()->data.node;
@@ -97,7 +118,7 @@ int main() {
         TEST_INFO (Test sorting "/hook/book" on /title);
         Expression *expr = evaluateExpr ("sort('/books/book','/title')");
         expr->setRoot (books);
-        NodeValueList *lst = expr->toNodeList ();
+        Sequence *lst = expr->toSequence ();
         NodeValueItem *itm = lst->first ();
         TEST_VERIFY (itm);
         if (itm) {
@@ -123,7 +144,7 @@ int main() {
         TEST_INFO (Test sorting "/hook/book" on @price);
         Expression *expr = evaluateExpr ("sort('/books/book','@price')");
         expr->setRoot (books);
-        NodeValueList *lst = expr->toNodeList ();
+        Sequence *lst = expr->toSequence ();
         NodeValueItem *itm = lst->first ();
         TEST_VERIFY (itm);
         if (itm) {
@@ -149,7 +170,7 @@ int main() {
         TEST_INFO (Test "/books/book[@price='5$'] | //book[@price='10p'] => 2");
         Expression *expr = evaluateExpr ("/books/book[@price='5$'] | //book[@price='10p']");
         expr->setRoot (books);
-        NodeValueList *lst = expr->toNodeList ();
+        Sequence *lst = expr->toSequence ();
         TEST_VERIFY (lst->length () == 2);
         NodeValueItem *itm = lst->first ();
         if (itm) {
@@ -163,6 +184,13 @@ int main() {
         delete lst;
         delete expr;
     }
+    {
+        TEST_INFO (Test "count(/books/book[@price='5$'] | //book[@price='10p']) => 2");
+        Expression *expr = evaluateExpr ("count(/books/book[@price='5$'] | //book[@price='10p'])");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toInt () == 2);
+        delete expr;
+    }
     {
         TEST_INFO (Test "string-join(//book[3]/title | //book[3]/@price, ' costs ')");
         Expression *expr = evaluateExpr ("string-join(//book[3]/title | //book[3]/@price, ' costs ')");
@@ -195,7 +223,7 @@ int main() {
         TEST_INFO (Test "//book/title[contains(.,'Tom')]");
         Expression *expr = evaluateExpr ("//book/title[contains(.,'Tom')]");
         expr->setRoot (books);
-        NodeValueList *lst = expr->toNodeList ();
+        Sequence *lst = expr->toSequence ();
         TEST_VERIFY (lst->length () == 2);
         NodeValueItem *itm = lst->first ();
         if (itm) {
@@ -221,6 +249,48 @@ int main() {
         TEST_VERIFY (expr->toString () == "Pygmalion");
         delete expr;
     }
+    {
+        TEST_INFO (Test substring-after(/books/book[2]/title, 'Tom') == 's Cabin');
+        Expression *expr = evaluateExpr ("substring-after(/books/book[2]/title, 'Tom')");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "'s Cabin");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test substring-before(/books/book[2]/title, 'Tom') == 'Uncle ');
+        Expression *expr = evaluateExpr ("substring-before(/books/book[2]/title, 'Tom')");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Uncle ");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test starts-with(/books/book[2]/title, 'Uncle') == true);
+        Expression *expr = evaluateExpr ("starts-with(/books/book[2]/title, 'Uncle')");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toBool ());
+        delete expr;
+    }
+    {
+        TEST_INFO (Test /books/book[starts-with(., 'Uncle')]/author == 'Harriet Beecher Stowe');
+        Expression *expr = evaluateExpr ("/books/book[starts-with(., 'Uncle')]/author");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Harriet Beecher Stowe");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test /books/book[starts-with('Uncle')]/author == 'Harriet Beecher Stowe');
+        Expression *expr = evaluateExpr ("/books/book[starts-with('Uncle')]/author");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toString () == "Harriet Beecher Stowe");
+        delete expr;
+    }
+    {
+        TEST_INFO (Test string-length(/books/book[1]/title) == 10);
+        Expression *expr = evaluateExpr ("string-length(/books/book[1]/title)");
+        expr->setRoot (books);
+        TEST_VERIFY (expr->toInt () == 10);
+        delete expr;
+    }
     books->document ()->dispose ();
     return 0;
 }
